-- =============================================================================================================
-- * Copyright (c) University of York
-- *
-- * File Name: DE3.vhd
-- *
-- * Author(s): M.Freeman
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY DE9000 IS
GENERIC (
  width           : INTEGER := 104;
  coeffiecient_0  : INTEGER := 0;
  coeffiecient_1  : INTEGER := 0;
  coeffiecient_2  : INTEGER := 0;
  coeffiecient_3  : INTEGER := 0;
  coeffiecient_4  : INTEGER := 0;
  coeffiecient_5  : INTEGER := 0;
  coeffiecient_6  : INTEGER := 0;
  coeffiecient_7  : INTEGER := 0;
  baud_rate       : INTEGER := 100;
  baud_rate_width : INTEGER := 8 );
PORT ( 
  clk : IN STD_LOGIC;
  clr : IN STD_LOGIC;
  result : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  tx : OUT STD_LOGIC);
END DE9000;

ARCHITECTURE DE9000_arch OF DE9000 IS

  --
  -- CONSTANTS
  --

 
  --
  -- SIGNALS
  --
  
  SIGNAL VCC : STD_LOGIC;
  SIGNAL GND : STD_LOGIC;
  SIGNAL GND_BUS : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  
  SIGNAL acc_0_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_1_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_2_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_3_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_4_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_5_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_6_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_7_add_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  
  SIGNAL acc_0_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_1_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_2_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_3_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_4_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_5_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_6_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  SIGNAL acc_7_reg_out : STD_LOGIC_VECTOR(width-1 DOWNTO 0);
   
  SIGNAL mux_out : STD_LOGIC_VECTOR(7 DOWNTO 0); 
  SIGNAL mux_sel : STD_LOGIC_VECTOR(3 DOWNTO 0); 
  
  SIGNAL dec_out : STD_LOGIC_VECTOR(7 DOWNTO 0); 
  SIGNAL dec_sel : STD_LOGIC_VECTOR(1 DOWNTO 0); 
  
  
  SIGNAL acc_en  : STD_LOGIC; 
  SIGNAL ser_wr  : STD_LOGIC; 
  SIGNAL tx_idle : STD_LOGIC; 
  
  SIGNAL cnt_ld : STD_LOGIC; 
  SIGNAL cnt_en : STD_LOGIC;    

  --
  -- COMPONENTS
  --
 
  --
  -- adder
  --
  
  COMPONENT adder
  GENERIC (
    width : INTEGER := 32 );
  PORT ( 
    din_a : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    din_b : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
    dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    zero : OUT STD_LOGIC;
    carry : OUT STD_LOGIC );
  END COMPONENT;
  
  --
  -- register
  --
  
  COMPONENT reg 
  GENERIC (
    width : INTEGER := 32;
    initial_value : INTEGER := 0 );
  PORT ( 
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    en : IN STD_LOGIC;
    rst : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0));
  END COMPONENT;
  
  --
  -- serial port
  --
  
  COMPONENT tx_serial_port
  GENERIC(
    BAUD_RATE : INTEGER := 1;
    BAUD_RATE_WIDTH : INTEGER := 2 );
  PORT (
    clk: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    wr: IN STD_LOGIC;
    data_i: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    tx: OUT STD_LOGIC;
    tx_idle: OUT STD_LOGIC;
    baud_test: OUT STD_LOGIC );
  END COMPONENT;  
  
  --
  -- multiplexer
  --
  
  COMPONENT mux 
  PORT ( 
    din_A  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_B  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_C  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_D  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_E  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_F  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_G  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_H  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_I  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_J  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_K  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_L  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    din_M  : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    dout   : OUT STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    sel    : IN STD_LOGIC_VECTOR( 3 DOWNTO 0 ) );
  END COMPONENT;
  
  --
  -- decoder
  --
  
  COMPONENT hex_char_decoder
  PORT ( 
    din : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    sel : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0) );
  END COMPONENT; 
  
  --
  -- counter
  --
  
  COMPONENT ld_up_cnt 
  GENERIC (
    width : INTEGER := 4 );
  PORT (
    clk: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    din: IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    dout: OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
    ld: IN STD_LOGIC;
    en: IN STD_LOGIC );
  END COMPONENT;
  
  --
  -- controller
  --
  
  COMPONENT controller
  PORT(
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    tx_idle : IN STD_LOGIC;
    acc_en  : OUT STD_LOGIC;
    ser_wr  : OUT STD_LOGIC;
    dec_sel : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cnt     : IN STD_LOGIC_VECTOR( 3 DOWNTO 0 );
    cnt_ld  : OUT STD_LOGIC;
    cnt_en  : OUT STD_LOGIC );
  END COMPONENT;  
 
BEGIN

  --
  -- SIGNAL
  --
  
  VCC     <= '1';
  GND     <= '0';
  GND_BUS <= (OTHERS=>'0');
  
  result <= acc_0_reg_out;
  
  --
  -- ACCUMULATOR 0
  --
  
  acc_0_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_0_reg_out, 
    din_b => acc_1_reg_out, 
    dout  => acc_0_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_0_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_0 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_0_add_out,
    dout => acc_0_reg_out );

  --
  -- ACCUMULATOR 1
  --
  
  acc_1_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_1_reg_out, 
    din_b => acc_2_reg_out, 
    dout  => acc_1_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_1_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_1 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_1_add_out,
    dout => acc_1_reg_out );

  --
  -- ACCUMULATOR 2
  --
  
  acc_2_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_2_reg_out, 
    din_b => acc_3_reg_out, 
    dout  => acc_2_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_2_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_2 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_2_add_out,
    dout => acc_2_reg_out );

  --
  -- ACCUMULATOR 3
  --
  
  acc_3_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_3_reg_out, 
    din_b => acc_4_reg_out, 
    dout  => acc_3_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_3_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_3 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_3_add_out,
    dout => acc_3_reg_out );
    
  --
  -- ACCUMULATOR 4
  --
  
  acc_4_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_4_reg_out, 
    din_b => acc_5_reg_out, 
    dout  => acc_4_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_4_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_4 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_4_add_out,
    dout => acc_4_reg_out );

  --
  -- ACCUMULATOR 5
  --
  
  acc_5_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_5_reg_out, 
    din_b => acc_6_reg_out, 
    dout  => acc_5_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_5_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_5 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_5_add_out,
    dout => acc_5_reg_out );

  --
  -- ACCUMULATOR 6
  --
  
  acc_6_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_6_reg_out, 
    din_b => acc_7_reg_out, 
    dout  => acc_6_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_6_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_6 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_6_add_out,
    dout => acc_6_reg_out );

  --
  -- ACCUMULATOR 7
  --
  
  acc_7_adder : adder
  GENERIC MAP(
    width => width )
  PORT MAP( 
    din_a => acc_7_reg_out, 
    din_b => GND_BUS, 
    dout  => acc_7_add_out,
    zero  => OPEN,
    carry => OPEN );

  acc_7_reg : reg 
  GENERIC MAP(
    width => width,
    initial_value => coeffiecient_7 )
  PORT MAP( 
    clk  => clk,
    clr  => clr, 
    en   => acc_en,
    rst  => GND,
    din  => acc_7_add_out,
    dout => acc_7_reg_out );
 
  --
  -- MUX
  --
  
  data_mux : mux 
  PORT MAP( 
    din_A  => acc_0_reg_out( 103 DOWNTO 96 ),
    din_B  => acc_0_reg_out( 95 DOWNTO 88 ),
    din_C  => acc_0_reg_out( 87 DOWNTO 80 ),
    din_D  => acc_0_reg_out( 79 DOWNTO 72 ),
    din_E  => acc_0_reg_out( 71 DOWNTO 64 ),
    din_F  => acc_0_reg_out( 63 DOWNTO 56 ),
    din_G  => acc_0_reg_out( 55 DOWNTO 48 ),
    din_H  => acc_0_reg_out( 47 DOWNTO 40 ),
    din_I  => acc_0_reg_out( 39 DOWNTO 32 ),
    din_J  => acc_0_reg_out( 31 DOWNTO 24 ),
    din_K  => acc_0_reg_out( 23 DOWNTO 16 ),
    din_L  => acc_0_reg_out( 15 DOWNTO 8 ),
    din_M  => acc_0_reg_out( 7 DOWNTO 0 ),
    dout   => mux_out,
    sel    => mux_sel );

  --
  -- HEX CHAR DECODE
  --
  
  decoder : hex_char_decoder 
  PORT MAP( 
    din  => mux_out,
    sel  => dec_sel,
    dout => dec_out );
  
  --
  -- SERIAL PORT
  --
  
  comms : tx_serial_port 
  GENERIC MAP(
    BAUD_RATE       => BAUD_RATE,
    BAUD_RATE_WIDTH => BAUD_RATE_WIDTH )
  PORT MAP(
    clk       => clk,
    clr       => clr,
    wr        => ser_wr,
    data_i    => dec_out,
    tx        => tx,
    tx_idle   => tx_idle,
    baud_test => OPEN );
    
  --
  -- COUNTER
  --
  
  count : ld_up_cnt 
  GENERIC MAP(
    width => 4 )
  PORT MAP(
    clk  => clk,
    clr  => clr,
    din  => GND_BUS(3 DOWNTO 0),
    dout => mux_sel,
    ld   => cnt_ld,
    en   => cnt_en );
 
  --
  -- CONTROLLER
  --
 
  ctl : controller 
  PORT MAP( 
    clk => clk,
    clr => clr,
    tx_idle => tx_idle,
    acc_en => acc_en,
    ser_wr => ser_wr,
    dec_sel => dec_sel,
    cnt    => mux_sel,
    cnt_ld => cnt_ld,
    cnt_en => cnt_en );

END DE9000_arch;
