-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: baud_clk.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: baud rate generator
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- * 
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;


ENTITY baud_clk IS
GENERIC(
  BAUD : INTEGER := 1;
  BAUD_WIDTH : INTEGER := 2 );
PORT (
  clk : IN STD_LOGIC;
  clr : IN STD_LOGIC;
  clk_en : OUT STD_LOGIC );
END baud_clk;

ARCHITECTURE baud_clk_arch OF baud_clk IS

  SIGNAL baud_count: STD_LOGIC_VECTOR(BAUD_WIDTH - 1 DOWNTO 0);

BEGIN

  -- rx sampling clock generator
  --
  PROCESS(clr, clk)
  BEGIN
    IF clr='1'
    THEN
      baud_count <= (OTHERS=>'0');
    ELSIF clk'event and clk ='1'
    THEN
      IF baud_count = conv_std_logic_vector(BAUD, BAUD_WIDTH)
      THEN
        baud_count <= (OTHERS=>'0');
      ELSE
        baud_count <= baud_count + 1;
      END IF;
    END IF;
  END PROCESS;

  -- comb logic output sig from rx counter
  --
  rxdecode: PROCESS(baud_count)
  BEGIN
    IF baud_count = conv_std_logic_vector(BAUD, BAUD_WIDTH)
    THEN
      clk_en <= '1';
    ELSE
      clk_en <= '0';
    END IF;
  END PROCESS;

END baud_clk_arch;
