-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_bit_cnt.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: serial port tx bit counter
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY tx_bit_cnt IS
PORT (
  clk: IN STD_LOGIC;
  baud: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  rst: IN STD_LOGIC;
  en: IN STD_LOGIC;
  max: OUT STD_LOGIC );
END tx_bit_cnt;

ARCHITECTURE tx_bit_cnt_arch OF tx_bit_cnt IS

  CONSTANT bit_count_max: STD_LOGIC_VECTOR(3 DOWNTO 0) := "1010";
  SIGNAL bit_count: STD_LOGIC_VECTOR(3 DOWNTO 0);

BEGIN

  -- tx bit count
  --
  bit_counter: PROCESS (clk, clr)
  BEGIN
    IF clr='1'
    THEN
      bit_count <= (OTHERS=>'0');
    ELSIF clk'event and clk='1'
    THEN
      IF baud ='1'
      THEN
        IF en='1'
        THEN
          IF rst='1'
          THEN
            bit_count <= (OTHERS=>'0');
          ELSE
            bit_count <= bit_count + 1;
          END IF;
        END IF;
      END IF;
    END IF;
  END PROCESS;

  -- comb logic output sig from bit counter
  --
  bitdecode: PROCESS(bit_count)
  BEGIN
    IF bit_count = bit_count_max
    THEN
      max <= '1';
    ELSE
      max <= '0';
    END IF;
  END PROCESS;

END tx_bit_cnt_arch;
