-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_handshake.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: handshake syncronisation
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tx_handshake IS
PORT (
  clk: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  wr_in: IN STD_LOGIC;
  wr_out: OUT STD_LOGIC;
  idle_in: IN STD_LOGIC;
  idle_out: OUT STD_LOGIC );
END tx_handshake;

ARCHITECTURE tx_handshake_arch OF tx_handshake IS

  TYPE state_type IS (S0, S1, S2);
  SIGNAL present_state, next_state: state_type;

BEGIN

  --
  -- signal buffers
  --

  --
  -- processes
  --

  sync: PROCESS(clk, clr)
  BEGIN
    IF clr='1'
    THEN
      present_state <= S0;
    ELSIF clk'event and clk='1'
    THEN
      present_state <= next_state;
    END IF;
  END PROCESS;

  comb: PROCESS(present_state, wr_in, idle_in)
  BEGIN
    idle_out <= '1';
    wr_out <= '0';

    CASE present_state IS

    -- wait for start
    --
    WHEN S0 =>

      IF wr_in='1' and idle_in='1'
      THEN
        idle_out <= '0';
        next_state <= S1;
      ELSE
        next_state <= S0;
      END IF;

    -- wait for idle_in to go low
    --
    WHEN S1 =>
      wr_out <= '1';
      idle_out <= '0';

      IF idle_in='1'
      THEN
        next_state <= S1;
      ELSE
        next_state <= S2;
      END IF;

    -- wait for idle_in to go high
    --
    WHEN S2 =>
      idle_out <= '0';

      IF idle_in='0'
      THEN
        next_state <= S2;
      ELSE
        next_state <= S0;
      END IF;

    --default condition
    --
    WHEN OTHERS =>
      next_state <= S0;

    END CASE;
  END PROCESS;

END tx_handshake_arch;
