-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_serial_port.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Generic tx serial port
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tx_serial_port IS
GENERIC(
  BAUD_RATE : INTEGER := 1;
  BAUD_RATE_WIDTH : INTEGER := 2 );
PORT (
  clk: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  wr: IN STD_LOGIC;
  data_i: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  tx: OUT STD_LOGIC;
  tx_idle: OUT STD_LOGIC;
  baud_test: OUT STD_LOGIC );
END tx_serial_port;

ARCHITECTURE tx_serial_port_arch OF tx_serial_port IS

  COMPONENT baud_clk
  GENERIC(
    BAUD : INTEGER := 1;
    BAUD_WIDTH : INTEGER := 2 );
  PORT (
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    clk_en : OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT tx_port
  PORT (
    clk:  IN STD_LOGIC;
    baud: IN STD_LOGIC;
    clr:  IN STD_LOGIC;
    wr:   IN STD_LOGIC;
    din:  IN STD_LOGIC_VECTOR (7 DOWNTO 0);
    tx:   OUT STD_LOGIC;
    idle: OUT STD_LOGIC );
  END COMPONENT;

  SIGNAL baud: STD_LOGIC;

BEGIN

  -- signal buffer
  --
  baud_test <= baud;

  -- baud rate generator
  --
  baud_clk_inst: baud_clk 
  GENERIC MAP(
    BAUD => BAUD_RATE,
    BAUD_WIDTH => BAUD_RATE_WIDTH )
  PORT MAP(
    clk => clk,
    clr => clr,
    clk_en => baud);

  -- tx port
  --
  tx_port_inst: tx_port PORT MAP(
    clk => clk,
    baud => baud,
    clr => clr,
    wr => wr,
    din => data_i,
    tx => tx,
    idle  => tx_idle );

END tx_serial_port_arch;
