-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_shift_reg.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: serial port tx parallel to serial shift register
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tx_shift_reg IS
PORT (
  clk: IN STD_LOGIC;
  baud: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  en: IN STD_LOGIC;
  ld: IN STD_LOGIC;
  data: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  tx: OUT STD_LOGIC);
END tx_shift_reg;

ARCHITECTURE tx_shift_reg_arch OF tx_shift_reg IS

  SIGNAL bit_reg: STD_LOGIC_VECTOR(9 DOWNTO 0);

BEGIN

  -- buffer
  --
  tx <= bit_reg(0);

  -- rx shift register
  --
  bit_shift_register: PROCESS(clr, clk)
  BEGIN
    IF clr='1'
    THEN
      bit_reg <= (OTHERS=>'1');
    ELSIF clk'event and clk='1'
    THEN
      IF baud ='1'
      THEN
        IF en='1'
        THEN
          IF ld='1'
          THEN
            bit_reg <= data & '0' & '1';
          ELSE
            bit_reg <= '1' & bit_reg(bit_reg'high downto 1);
          END IF;
        END IF;
      END IF;
    END IF;
  END PROCESS;

END tx_shift_reg_arch;
