-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_sync.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: serial port tx synchronized clock
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY tx_sync IS
PORT (
  clk: IN STD_LOGIC;
  baud: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  rst: IN STD_LOGIC;
  sPnt: OUT STD_LOGIC );
END tx_sync;

ARCHITECTURE tx_sync_arch OF tx_sync IS

  CONSTANT tx_clk_count_max: STD_LOGIC_VECTOR(3 DOWNTO 0) := "1111";
  SIGNAL tx_clk_count: STD_LOGIC_VECTOR(3 DOWNTO 0);

BEGIN

  sync: PROCESS(clr, clk)
  BEGIN
    IF clr='1'
    THEN
      tx_clk_count <= (OTHERS=>'0');
    ELSIF clk'event and clk ='1'
    THEN
      IF rst='1'
      THEN
        tx_clk_count <= (OTHERS=>'0');
      ELSIF baud='1'
      THEN
        IF tx_clk_count = tx_clk_count_max
        THEN
          tx_clk_count <= (OTHERS=>'0');
        ELSE
          tx_clk_count <= tx_clk_count + 1;
        END IF;
      END IF;
    END IF;
  END PROCESS;

  -- comb logic output sig from rx counter
  --
  txdecode: PROCESS(tx_clk_count)
  BEGIN
    IF tx_clk_count = tx_clk_count_max
    THEN
      sPnt <= '1';
    ELSE
      sPnt <= '0';
    END IF;
  END PROCESS;

END tx_sync_arch;

