-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: DE3_tb.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Difference Engine Mark 3 testbench
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;
USE ieee.numeric_std.ALL;
 
ENTITY DE3_tb IS
END DE3_tb;
 
ARCHITECTURE behavior OF DE3_tb IS 
 
  --
  -- Component Declaration for the Unit Under Test (UUT)
  -- 
  
  COMPONENT DE3
  GENERIC (
    width           : INTEGER := 32;
    coeffiecient_0  : INTEGER := 0;
    coeffiecient_1  : INTEGER := 0;
    coeffiecient_2  : INTEGER := 0;
    coeffiecient_3  : INTEGER := 0;
    coeffiecient_4  : INTEGER := 0;
    coeffiecient_5  : INTEGER := 0;
    coeffiecient_6  : INTEGER := 0;
    coeffiecient_7  : INTEGER := 0;
    BAUD_RATE       : INTEGER := 100;
    BAUD_RATE_WIDTH : INTEGER := 8 );
  PORT(
    clk : IN  STD_LOGIC;
    clr : IN  STD_LOGIC;
    result : OUT  STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    tx : OUT  STD_LOGIC );
  END COMPONENT;
    
  --
  --Constants
  --
  
  CONSTANT WIDTH : INTEGER := 104;
  
  --
  --Inputs
  --
  
  SIGNAL clk : STD_LOGIC := '0';
  SIGNAL clr : STD_LOGIC := '0';

  --
 	--Outputs
  --
  
  SIGNAL result : STD_LOGIC_VECTOR(WIDTH-1 DOWNTO 0);
  SIGNAL tx : STD_LOGIC;

  --
  -- Clock period definitions
  --
  
  CONSTANT clk_period : time := 100 ns;
 
BEGIN

  --
	-- Instantiate the Unit Under Test (UUT)
  --
  
  uut: DE3 
  GENERIC MAP(
    width           => WIDTH,
    coeffiecient_0  => 0,
    coeffiecient_1  => 1,
    coeffiecient_2  => 6,
    coeffiecient_3  => 6,
    coeffiecient_4  => 0,
    coeffiecient_5  => 0,
    coeffiecient_6  => 0,
    coeffiecient_7  => 0,
    BAUD_RATE       => 2,
    BAUD_RATE_WIDTH => 8 )
  PORT MAP (
    clk => clk,
    clr => clr,
    result => result,
    tx => tx );

  --
  -- Clock process definitions
  --
  
  clk_process :PROCESS
  BEGIN
    clk <= '0';
	  WAIT FOR clk_period / 2;
		clk <= '1';
	  WAIT FOR clk_period / 2;
  END PROCESS;
 
  --
  -- Reset process definitions
  --
  
  clr_process :PROCESS
  BEGIN
    clr <= '1';
	  WAIT FOR clk_period*10;
		clr <= '0';
	  WAIT;
  END PROCESS;

END;
