-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: DE3_Wrapper.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Difference Engine Mark 3 FPGA Wrapper
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY DE3_Wrapper IS
GENERIC(
  width : INTEGER := 104 );
PORT ( 
  clk : IN STD_LOGIC;
  clr : IN STD_LOGIC;
  tx  : OUT STD_LOGIC_VECTOR(3 DOWNTO 0) );
END DE3_Wrapper;

ARCHITECTURE DE3_Wrapper_arch OF DE3_Wrapper IS

  --
  -- CONSTANTS
  --
  
  CONSTANT MAX : INTEGER := 3;
  
  --
  -- SIGNALS
  --
  
  SIGNAL clr_n  : STD_LOGIC;
  SIGNAL result : STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
  
  --
  -- COMPONENTS
  --
  
  COMPONENT DE3 
  GENERIC (
    width           : INTEGER := 104;
    coeffiecient_0  : INTEGER := 0;
    coeffiecient_1  : INTEGER := 0;
    coeffiecient_2  : INTEGER := 0;
    coeffiecient_3  : INTEGER := 0;
    coeffiecient_4  : INTEGER := 0;
    coeffiecient_5  : INTEGER := 0;
    coeffiecient_6  : INTEGER := 0;
    coeffiecient_7  : INTEGER := 0;
    BAUD_RATE       : INTEGER := 100;
    BAUD_RATE_WIDTH : INTEGER := 8 );
  PORT ( 
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    result : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    tx : OUT STD_LOGIC);
  END COMPONENT;
  
BEGIN

  --
  -- SIGNALS
  --
  
  clr_n <= not clr;
  
  --
  -- COMPONENTS
  --
  
  -- BAUD_RATE       => 52
  -- BAUD_RATE_WIDTH => 6
  
  -- BAUD_RATE       => 65
  -- BAUD_RATE_WIDTH => 7
  
  de_array: FOR i in 0 TO MAX GENERATE
  BEGIN
		DE3_Inst : DE3 
		GENERIC MAP(
			width           => width,
			coeffiecient_0  => 0,
			coeffiecient_1  => 1,
			coeffiecient_2  => 6,
			coeffiecient_3  => 6,
			coeffiecient_4  => 0,
			coeffiecient_5  => 0,
			coeffiecient_6  => 0,
			coeffiecient_7  => 0,
			BAUD_RATE       => 65,
			BAUD_RATE_WIDTH => 7 )
		PORT MAP( 
			clk    => clk,
			clr    => clr_n,
			result => OPEN,
			tx     => tx(i) );
	END GENERATE;
	 

END DE3_Wrapper_arch;
