-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: adder.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Generic Adder
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY adder IS
GENERIC (
  width : INTEGER := 32 );
PORT ( 
  din_a : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  din_b : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
  dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  zero : OUT STD_LOGIC;
  carry : OUT STD_LOGIC );
END adder;

ARCHITECTURE adder_arch OF adder IS

 CONSTANT zero_value : STD_LOGIC_VECTOR(width-1 DOWNTO 0) := (OTHERS=>'0');
 
 SIGNAL din_a_int : STD_LOGIC_VECTOR(width DOWNTO 0);
 SIGNAL din_b_int : STD_LOGIC_VECTOR(width DOWNTO 0);
 SIGNAL dout_int : STD_LOGIC_VECTOR(width DOWNTO 0);
 
BEGIN

  -- signals 
  din_a_int <= '0' & din_a;
  din_b_int <= '0' & din_b;  
  dout <= dout_int(width-1 DOWNTO 0);
  carry <= dout_int(width);
  
  -- adder
  generic_adder : PROCESS(din_a_int, din_b_int)
  BEGIN
    dout_int <= din_a_int + din_b_int;
  END PROCESS;

  -- zero
  zero_test : PROCESS(dout_int)
  BEGIN
    IF dout_int(width-1 DOWNTO 0) = zero_value
    THEN
      zero <= '1';
    ELSE
      zero <= '0';
    END IF;
  END PROCESS;  

END adder_arch;
