-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: controller.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: DE3 controller
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY controller IS
PORT ( 
  clk : IN STD_LOGIC;
  clr : IN STD_LOGIC;
  tx_idle : IN STD_LOGIC;
  acc_en  : OUT STD_LOGIC;
  ser_wr  : OUT STD_LOGIC;
  dec_sel : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
  cnt : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
  cnt_ld : OUT STD_LOGIC;
  cnt_en : OUT STD_LOGIC );
END controller;

ARCHITECTURE controller_arch OF controller IS

  TYPE state IS ( S0, S1, S1w, S2, S2w, S3, S4, S4w, S5 );
  SIGNAL present_state, next_state : state;

BEGIN

  sync: PROCESS(clk, clr)
  BEGIN
    IF clr='1'
    THEN
      present_state <= S0;
    ELSIF clk='1' and clk'event
    THEN
      present_state <= next_state;
    END IF;
  END PROCESS;

  comb: PROCESS( present_state, tx_idle, cnt )
  BEGIN
    acc_en  <= '0';
    ser_wr  <= '0';
    dec_sel <= "00";
    cnt_ld  <= '0';
    cnt_en  <= '0';
    
    CASE present_state IS
      WHEN S0 =>  
        cnt_ld  <= '1';        -- zero byte count
        cnt_en  <= '1';
        
        IF tx_idle = '1'
        THEN
          next_state <= S1;    -- if serial port ready start
        ELSE
          next_state <= S0;    -- if not wait
        END IF; 
        
      WHEN S1 => 
        dec_sel <= "01";        -- select high nibble
        ser_wr  <= '1';
        next_state <= S1w; 
        
      WHEN S1w =>   
        IF tx_idle = '1'       -- wait for UART to tx
        THEN
          next_state <= S2; 
        ELSE
          next_state <= S1w;
        END IF; 
        
      WHEN S2 => 
        dec_sel <= "00";        -- select low nibble     
        ser_wr  <= '1';
        next_state <= S2w; 
        
      WHEN S2w =>   
        IF tx_idle = '1'       -- wait for UART to tx
        THEN
          next_state <= S3; 
        ELSE
          next_state <= S2w;
        END IF;
        
      WHEN S3 =>
        cnt_en <= '1';          -- increment byte count
        
        IF cnt = "1100"
        THEN
          next_state <= S4;
        ELSE
          next_state <= S1;
        END IF; 
        
      WHEN S4 =>
        dec_sel <= "10";        -- select CR  
        ser_wr  <= '1';
        next_state <= S4w; 
        
      WHEN S4w =>   
        IF tx_idle = '1'       -- wait for UART to tx
        THEN
          next_state <= S5; 
        ELSE
          next_state <= S4w;
        END IF;
		  
      WHEN S5 =>  
        acc_en <= '1';          -- clock acc
        next_state <= S1; 
        
      WHEN OTHERS =>
        next_state <= S0;

    END CASE;
  END PROCESS;

END controller_arch;
