-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: hex_char_decoder.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Hex character decoder
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *  
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY hex_char_decoder IS
PORT ( 
  din : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  sel : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0) );
END hex_char_decoder;

ARCHITECTURE hex_char_decoder_arch OF hex_char_decoder IS
BEGIN

  decoder : PROCESS( din, sel )
  BEGIN
    IF sel(1)='0'
	 THEN
      IF sel(0)='0'
      THEN
        CASE ( din(3 DOWNTO 0) ) IS
          WHEN "0000" => dout <= "00110000";  -- 0x30 - "0"
          WHEN "0001" => dout <= "00110001";  -- 0x31 - "1"  
          WHEN "0010" => dout <= "00110010";  -- 0x32 - "2"
          WHEN "0011" => dout <= "00110011";  -- 0x33 - "3"
          WHEN "0100" => dout <= "00110100";  -- 0x34 - "4"
          WHEN "0101" => dout <= "00110101";  -- 0x35 - "5"  
          WHEN "0110" => dout <= "00110110";  -- 0x36 - "6"
          WHEN "0111" => dout <= "00110111";  -- 0x37 - "7"
          WHEN "1000" => dout <= "00111000";  -- 0x38 - "8"
          WHEN "1001" => dout <= "00111001";  -- 0x39 - "9"  
          WHEN "1010" => dout <= "01000001";  -- 0x41 - "A"
          WHEN "1011" => dout <= "01000010";  -- 0x42 - "B"
          WHEN "1100" => dout <= "01000011";  -- 0x43 - "C"
          WHEN "1101" => dout <= "01000100";  -- 0x44 - "D"  
          WHEN "1110" => dout <= "01000101";  -- 0x45 - "E"
          WHEN "1111" => dout <= "01000110";  -- 0x46 - "F"
          WHEN OTHERS => dout <= "11111111";  -- 0x30 - "0"
        END CASE;
      ELSE
        CASE ( din(7 DOWNTO 4) ) IS
          WHEN "0000" => dout <= "00110000";  -- 0x30 - "0"
          WHEN "0001" => dout <= "00110001";  -- 0x31 - "1"  
          WHEN "0010" => dout <= "00110010";  -- 0x32 - "2"
          WHEN "0011" => dout <= "00110011";  -- 0x33 - "3"
          WHEN "0100" => dout <= "00110100";  -- 0x34 - "4"
          WHEN "0101" => dout <= "00110101";  -- 0x35 - "5"  
          WHEN "0110" => dout <= "00110110";  -- 0x36 - "6"
          WHEN "0111" => dout <= "00110111";  -- 0x37 - "7"
          WHEN "1000" => dout <= "00111000";  -- 0x38 - "8"
          WHEN "1001" => dout <= "00111001";  -- 0x39 - "9"  
          WHEN "1010" => dout <= "01000001";  -- 0x41 - "A"
          WHEN "1011" => dout <= "01000010";  -- 0x42 - "B"
          WHEN "1100" => dout <= "01000011";  -- 0x43 - "C"
          WHEN "1101" => dout <= "01000100";  -- 0x44 - "D"  
          WHEN "1110" => dout <= "01000101";  -- 0x45 - "E"
          WHEN "1111" => dout <= "01000110";  -- 0x46 - "F"
          WHEN OTHERS => dout <= "11111111";  -- 0x30 - "0"
        END CASE;
      END IF;
    ELSE
      IF sel(0)='0'
      THEN
		  dout <= "00001101"; -- CR
		ELSE  
		  dout <= "00001010"; -- LF
      END IF;
    END IF;		
  END PROCESS;  
END hex_char_decoder_arch;
