-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: ld_up_cnt.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: loadable counter
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY ld_up_cnt IS
GENERIC (
  width : INTEGER := 32 );
PORT (
  clk: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  din: IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
  dout: OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
  ld: IN STD_LOGIC;
  en: IN STD_LOGIC );
END ld_up_cnt;

ARCHITECTURE ld_up_cnt_arch OF ld_up_cnt IS

  SIGNAL count : STD_LOGIC_VECTOR(width-1 DOWNTO 0);

BEGIN

  -- signal buffers
  --
  dout <= count;
  
  -- counter sync part
  --
  counter: PROCESS (clk, clr)
  BEGIN
    IF clr='1'
    THEN
      count <= (OTHERS=>'0');
    ELSIF clk'event and clk='1'
    THEN
      IF en='1'
      THEN
        IF ld='1'
        THEN
          count <= din;
        ELSE
          count <= count + 1;
        END IF;
      END IF;
    END IF;
  END PROCESS;

END ld_up_cnt_arch;

