-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: tx_state_mac.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: serial port tx state machine
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tx_state_mac IS
PORT (
  clk: IN STD_LOGIC;
  baud: IN STD_LOGIC;
  clr: IN STD_LOGIC;
  sPnt: IN STD_LOGIC;
  bMax: IN STD_LOGIC;
  wr: IN STD_LOGIC;
  txRst: OUT STD_LOGIC;
  bCen: OUT STD_LOGIC;
  bRst: OUT STD_LOGIC;
  bSht: OUT STD_LOGIC;
  bLd: OUT STD_LOGIC;
  idle: OUT STD_LOGIC );
END tx_state_mac;

ARCHITECTURE tx_state_mac_arch OF tx_state_mac IS

  TYPE state_type IS (S0, S1, S2);
  SIGNAL present_state, next_state: state_type;

BEGIN

  -- tx state machine
  --
  sync: PROCESS(clk, clr)
  BEGIN
    IF clr='1'
    THEN
      present_state <= S0;
    ELSIF clk'event and clk='1'
    THEN
      IF baud='1'
      THEN
        present_state <= next_state;
      END IF;
    END IF;
  END PROCESS;

  txidle: PROCESS(present_state)
  BEGIN
    IF present_state = S0
    THEN
      idle <= '1';
    ELSE
      idle <= '0';
    END IF;
  END PROCESS;

  comb: PROCESS(sPnt, bMax, wr, present_state)
  BEGIN
    txRst <= '0';
    bCen   <= '0';
    bRst  <= '0';
    bSht  <= '0';
    bLd  <= '0';

    CASE present_state IS

    -- wait for data
    --
    WHEN S0 =>
      txRst <= '1';
      bCen  <= '1';
      bRst  <= '1';

      IF wr='0'
      THEN
        next_state <= S0;
      ELSE
        next_state <= S1;
      END IF;

    -- store data into shift register
    --
    WHEN S1 =>
      bSht  <= '1';
      bLd <= '1';

      next_state <= S2;

    -- shift tx data
    --
    WHEN S2 =>

      IF sPnt ='1'
      THEN
        bCen  <= '1';
        bSht  <= '1';
        
        IF bMax='1'
        THEN
          next_state <= S0;  -- finished tx of data
        ELSE
          next_state <= S2;  -- shift data out of shift register
        END IF;
      ELSE
        next_state <= S2;  -- wait for sample point
      END IF;

    --default condition
    --
    WHEN OTHERS =>

      next_state <= S0;

    END CASE;
  END PROCESS;

END tx_state_mac_arch;

